/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "FSysUtil.h"
#include "QDebug.h"
#include "../Util.h"
#include "MountUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

#include <unistd.h>
#include <stdio.h>

namespace Pline 
{

static Glib::ustring zstMkFSys[] = {	
	"ext2",			"/sbin/mkfs.ext2",			"-L \"%s\"",
	"ext3",			"/sbin/mkfs.ext3",			"-L \"%s\"",
	"ext4",			"/sbin/mkfs.ext4",			"-L \"%s\"",
	"fat16",		"/sbin/mkfs.msdos",			"-n \"%s\" -F \"16\"",
	"fat32",		"/sbin/mkfs.vfat",			"-n \"%s\" -F \"32\"",
	"hfs",			"/sbin/mkfs.hfs",			"-h -v \"%s\"",
	"hfsplus",		"/sbin/mkfs.hfsplus",		"-v \"%s\"",
	"ntfs",			"/sbin/mkfs.ntfs",			"-f -L \"%s\"",
	"reiserfs",		"/sbin/mkfs.reiserfs",		"-q -l \"%s\"",
	"xfs",			"/sbin/mkfs.xfs",			"-f -L \"%s\"",
	"",				"",							"",
};

static Glib::ustring zstEnlargeFSys[] = {	
	"ext2",			"/sbin/resize2fs",
	"ext3",			"/sbin/resize2fs",
	"ext4",			"/sbin/resize2fs",
	"reiserfs",		"/sbin/resize_reiserfs",
	"xfs",			"/usr/sbin/xfs_growfs",
	"",				"",
};

//==============================================================================

FSysUtil::FSysUtil()
{
}

//==============================================================================

FSysUtil::~FSysUtil()
{
}

//==============================================================================

bool FSysUtil::enlarge(	const Glib::ustring& stDev,
						const Glib::ustring& stDir,
						const Glib::ustring& stFSys,
						Glib::ustring *pstErrMsg)
{
	if (!Util::isDevice(stDev, pstErrMsg))
		return false;
	
	if (!Util::isDirectory(stDir, pstErrMsg))
		return false;
		
	if (stFSys == "ext2"
		|| stFSys == "ext3"
		|| stFSys == "ext4")
	{
		return enlarge_ext(stDev, pstErrMsg);
	}
	else if (stFSys == "raiserfs")
		return enlarge_raiserfs(stDev, pstErrMsg);
	else if (stFSys == "xfs")
		return enlarge_xfs(stDir, pstErrMsg);
	else
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid filesystem.");
			
		return false;
	}
}

//==============================================================================

bool FSysUtil::enlarge_ext(	const Glib::ustring& stDev,
							Glib::ustring *pstErrMsg)
{
	Glib::ustring stChkCmd;
	Glib::ustring stRszCmd;
	Glib::ustring stOutBuf;

	// Check fsys
	stChkCmd = Util::compose("/sbin/fsck -fp %s", stDev.c_str());
	
	if (!Util::exec(stChkCmd, &stOutBuf))		
		return false;

	Glib::usleep(1000*3500);
	
	// Resize fsys
	stRszCmd = Util::compose("/sbin/resize2fs %s", stDev.c_str());
	
	if (!Util::exec(stRszCmd, &stOutBuf))
	{
		if (pstErrMsg)
			*pstErrMsg = stOutBuf;
		
		return false;
	}
	
	return true;
}

//==============================================================================

bool FSysUtil::enlarge_raiserfs(const Glib::ustring& stDev,
								Glib::ustring *pstErrMsg)
{
	Glib::ustring stRszCmd;
	Glib::ustring stOutBuf;
	
	// Resize fsys
	stRszCmd = Util::compose("/sbin/resize_reis1erfs -fq %s", stDev.c_str());
	
	if (!Util::exec(stRszCmd, &stOutBuf))
	{
		if (pstErrMsg)
			*pstErrMsg = stOutBuf;
		
		return false;
	}
	
	return true;
}

//==============================================================================

bool FSysUtil::enlarge_xfs(	const Glib::ustring& stDir,
							Glib::ustring *pstErrMsg)
{
	Glib::ustring stRszCmd;
	Glib::ustring stOutBuf;
		
	// Resize fsys
	stRszCmd = Util::compose("/usr/sbin/xfs_growfs %s", stDir.c_str());
	
	if (!Util::exec(stRszCmd, &stOutBuf))
	{
		if (pstErrMsg)
			*pstErrMsg = stOutBuf;
		
		return false;
	}
	
	return true;
}
							
//==============================================================================

bool FSysUtil::format(	const Glib::ustring& stDev,
						const Glib::ustring& stVolumeLabel,
						const Glib::ustring& stFSys,
						Glib::ustring *pstErrMsg)
{
	// validate data
	if (!Util::isDevice(stDev, pstErrMsg))
		return false;
	
	if (stFSys.empty())
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid filesystem.");
			
		return false;
	}
	
	// get fsys command
	Glib::ustring stCmd;
	
	if (!getMkFSysCommand(stFSys, &stCmd))
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("Filesystem %s not supported."), stFSys.c_str());
			
		return false;
	}
	
	// volume label
	Glib::ustring stVolumeLabelEscaped = getEscapedVolumeLabel(stVolumeLabel);
	
	// get args
	Glib::ustring stArg;
	
	assert(getMkFSysArg(stFSys, &stArg));
	
	// format
	stArg = Util::compose(stArg.c_str(), stVolumeLabelEscaped.c_str());
	stCmd = Util::compose(	"%s %s %s", 
							stCmd.c_str(), 
							stArg.c_str(),
							stDev.c_str());
	
	return Util::exec(stCmd, pstErrMsg);
}

//==============================================================================

void FSysUtil::getEnlargeFSysList(std::vector<Glib::ustring> *pvecFSys)
{
	int xi = 0;
	
	while (true)
	{
		Glib::ustring stFSysName = zstEnlargeFSys[xi];
		Glib::ustring stFSysCmd = zstEnlargeFSys[xi+1];
		
		if (stFSysName.empty() || stFSysCmd.empty())
			break;
		
		if (Util::isFileExist(stFSysCmd))
			pvecFSys->push_back(stFSysName);
		
		xi += 2;
	}
}

//==============================================================================

const Glib::ustring FSysUtil::getEscapedVolumeLabel(const Glib::ustring& stVolumeLabel)
{
	Glib::ustring stVolumeLabel2 = stVolumeLabel;
	Glib::ustring stVolumeLabelEscaped;
	
	for (	Glib::ustring::iterator ite = stVolumeLabel2.begin();
			ite != stVolumeLabel2.end();
			++ite)
	{
		gunichar ch = *ite;
		
		if ((ch >= 0x30 && ch <= 0x39) // 0-9
			|| (ch >= 0x41 && ch <= 0x5a) // a-z
			|| (ch >= 0x61 && ch <= 0x7a)) // A-Z
		{
			stVolumeLabelEscaped +=  ch;
			
			if (stVolumeLabelEscaped.size() >= 8)
				break;
		}
	}
	
	return stVolumeLabelEscaped;
}

//==============================================================================

bool FSysUtil::getMkFSysArg(const Glib::ustring& stFSys,
							Glib::ustring *pstArg)
{
	assert(pstArg);
	
	int xi = 0;
	
	while (true)
	{
		Glib::ustring stFSysName = zstMkFSys[xi];
		Glib::ustring stFSysArg = zstMkFSys[xi+2];
		
		if (stFSysName.empty() || stFSysArg.empty())
			break;
		
		if (stFSys == stFSysName)
		{
			*pstArg = stFSysArg;
			return true;
		}
		
		xi += 3;
	}
	
	return false;
}

//==============================================================================

bool FSysUtil::getMkFSysCommand(const Glib::ustring& stFSys,
								Glib::ustring *pstCmd)
{
	assert(pstCmd);
	
	int xi = 0;
	
	while (true)
	{
		Glib::ustring stFSysName = zstMkFSys[xi];
		Glib::ustring stFSysCmd = zstMkFSys[xi+1];
		
		if (stFSysName.empty() || stFSysCmd.empty())
			break;
		
		if (stFSys == stFSysName)
		{
			*pstCmd = stFSysCmd;
			return true;
		}
		
		xi += 3;
	}
	
	return false;
}

//==============================================================================

void FSysUtil::getMkFSysList(std::vector<Glib::ustring> *pvecFSys)
{
	int xi = 0;
	
	while (true)
	{
		Glib::ustring stFSysName = zstMkFSys[xi];
		Glib::ustring stFSysCmd = zstMkFSys[xi+1];
		
		if (stFSysName.empty() || stFSysCmd.empty())
			break;
		
		if (Util::isFileExist(stFSysCmd))
			pvecFSys->push_back(stFSysName);
		
		xi += 3;
	}
}

//==============================================================================

} // Pline
