/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "InitVolumePline.h"
#include "QDebug.h"
#include "../Util.h"

#include "MountPline.h"

#include "CloneData.h"
#include "CopyFilter.h"
#include "CryptFilter.h"
#include "LoopFilter.h"
#include "MountFilter.h"
#include "MkFSysFilter.h"
#include "MkCryptFilter.h"

#include <iostream>
#include <iomanip>
#include <glibmm/i18n.h>
#include <assert.h>

namespace Pline
{

//==============================================================================

InitVolumePline::InitVolumePline( 	const eDataMode eDataModeItem,
									VolumeConfig *poNewVolumeConfig,
									VolumeConfig *poOldVolumeConfig,
									const Glib::ustring& stCipher,
									const Glib::ustring& stCBC,
									const Glib::ustring& stKeySize)
: ProcessBasePline(eDataModeItem, poNewVolumeConfig)
{
	assert(goVolumeConfig.getMountData());
	
	// init mount pline
	gpoMountPline = new MountPline(eDataModeItem, poOldVolumeConfig);
	gpoMountPline->signal_req_data().connect(
			sigc::mem_fun(*this, &InitVolumePline::onMountPlineReqData));
			
	giPctDone = 0;
	
	gbCryptFormat = false;
	gbCryptOpen = false;
	gbMkFSys = false;
	
	gpoCryptDetachFilter = 0;
	gpoLoopDetachFilter = 0;
	
	bool bUseLoop =
		!(Util::isDevice(goVolumeConfig.getData().getSrc()));
		
	bool bUseCrypt = 
		(goVolumeConfig.getCryptData() && goVolumeConfig.getCryptData()->getIsEncrypted());
	
	// 
	// Set FSys filter
	// 
	Data *poLoopDestData = 0;
	MountData* poMountData = goVolumeConfig.getMountData();
	
	poLoopDestData = 
		(bUseCrypt) ? (Data*)goVolumeConfig.getCryptData() : (Data*)goVolumeConfig.getMountData();
	
	//
	// Loop open
	//
	if (bUseLoop)
	{
		LoopFilter *poLoopAttachFilter = new LoopFilter;
	
		// set data from config
		LoopData* poLoopData = (LoopData*)goVolumeConfig.getDataPtr();
			
		// set filter data
		poLoopAttachFilter->setSrcData(poLoopData);
		poLoopAttachFilter->setDestData(poLoopDestData);
	
		// set filter run mode
		poLoopAttachFilter->setRunMode(MountBaseFilter::kRunAttach);
	
		// add filter to pline
		addFilter(poLoopAttachFilter);
	}
	
	//
	// Encrypt
	//
	if (bUseCrypt)
	{
		//
		// format
		//
		MkCryptFilter *poMkCryptFilter = new MkCryptFilter;
		
		// set filter data
		poMkCryptFilter->setSrcData((CryptData*)poLoopDestData);
		poMkCryptFilter->setCipher(stCipher);
		poMkCryptFilter->setCBC(stCBC);
		poMkCryptFilter->setKeySize(stKeySize);
		
		// add filter to pline
		addFilter(poMkCryptFilter);
		
		//
		// open
		//
		CryptFilter *poCryptAttachFilter = new CryptFilter;
		
		// set filter data
		poCryptAttachFilter->setSrcData((CryptData*)poLoopDestData);
		poCryptAttachFilter->setDestData(poMountData);
		
		// set filter run mode
		poCryptAttachFilter->setRunMode(MountBaseFilter::kRunAttach);
		
		// add filter to pline
		addFilter(poCryptAttachFilter);
	}
	
	//
	// Create FSys
	//
	MkFSysFilter *poMkFSysFilter = new MkFSysFilter;
	
	// set data from config
	poMkFSysFilter->setSrcData(poMountData);
	
	// add filter to pline
	addFilter(poMkFSysFilter);
	
	//
	// Encrypt close
	//
	if (bUseCrypt)
	{
		gpoCryptDetachFilter = new CryptFilter;
		
		// set filter data
		gpoCryptDetachFilter->setSrcData((CryptData*)poLoopDestData);
		gpoCryptDetachFilter->setDestData(poMountData);
		
		// set filter run mode
		gpoCryptDetachFilter->setRunMode(MountBaseFilter::kRunDetach);
		
		// add filter to pline
		addFilter(gpoCryptDetachFilter);
	}
	
	//
	// Loop close
	//
	if (bUseLoop)
	{
		gpoLoopDetachFilter = new LoopFilter;
	
		// set data from config
		LoopData* poLoopData = (LoopData*)goVolumeConfig.getDataPtr();
		
		// set filter data
		gpoLoopDetachFilter->setSrcData(poLoopData);
		gpoLoopDetachFilter->setDestData(poLoopDestData);
	
		// set filter run mode
		gpoLoopDetachFilter->setRunMode(MountBaseFilter::kRunDetach);
	
		// add filter to pline
		addFilter(gpoLoopDetachFilter);
	}
}

//==============================================================================

InitVolumePline::~InitVolumePline()
{
	delete gpoMountPline;
}

//==============================================================================

void InitVolumePline::askData()
{
	(gpoReqFilter) ?
		ProcessBasePline::askData() : gpoMountPline->askData();
}

//==============================================================================

const Glib::ustring InitVolumePline::getInfoMessage()
{
	if (gbCryptFormat)
	{
		giPctDone = 0;
		return _("Encrypting volume...");
	}
	else if (gbCryptOpen)
	{
		giPctDone = 20;
		return _("Opening secured volume...");
	}
	else if (gbMkFSys)
	{
		giPctDone = 50;
		return _("Initializing filesystem, please wait...");
	}
	
	return _("Initializing volume...");
}

//==============================================================================

bool InitVolumePline::onFilterPreRun(Filter *poFilter)
{
	gbCryptFormat = (poFilter->getFilterName() == "MkCrypt");
	gbCryptOpen = (poFilter->getFilterName() == "Crypt" 
					&& poFilter != gpoCryptDetachFilter);
	gbMkFSys = (poFilter->getFilterName() == "MkFSys");
	
	return ProcessBasePline::onFilterPreRun(poFilter);
}

//==============================================================================

void InitVolumePline::onMountPlineReqData()
{
	m_signal_req_data.emit();
}

//==============================================================================

bool InitVolumePline::run()
{	
	initError();
	
	// manage completed signal locally
	Pline::skipCompletedSignal(true);

	bool bMountPlineInUse = gpoMountPline->isInUse();
	
	// eject mount pline
	if (bMountPlineInUse)
	{
		gpoMountPline->setRunMode(MountPline::kRunEject);
		
		if (!gpoMountPline->run())
		{
			gtLastError.stErrMsg = _("Could not eject volume.");
			
			m_signal_completed(false);
			return false;
		}
	}
		
	bool bRet = Pline::run();
	
	if (!bRet)
	{
		if (gpoCryptDetachFilter)
			gpoCryptDetachFilter->run();
		
		if (gpoLoopDetachFilter)
			gpoLoopDetachFilter->run();
	}
	
	m_signal_completed(bRet);
	
	return bRet;
}

//==============================================================================

} // Pline	
