/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MountFilter.h"
#include "MountUtil.h"
#include "../QDebug.h"
#include "../eMountSys.h"
#include "../UtilSh.h"
#include "../VolumeDialog.h"
#include "../QConfig.h"
#include "../Util.h"

#include <assert.h>
#include <sys/mount.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>

namespace Pline
{

//==============================================================================

MountFilter::MountFilter()
: MountBaseFilter()
{
	gpoMountData = 0;
	gpoSrcPathData = 0;
}

//==============================================================================

MountFilter::~MountFilter()
{
}

//==============================================================================

void MountFilter::askData()
{	
	if (eMountSys::getInstance().getRunMode() != eMountSys::kRunGui)
		return Filter::askData();
	
	if (!isValidMountPoint())
	{
		VolumeDialog::tConfig tConfigItem;
		VolumeDialog::initConfig(tConfigItem);
		
		tConfigItem.bAutoRun = false;
		tConfigItem.bWaitData = false;
		tConfigItem.bAskSelectDir = true;
		tConfigItem.stSelectDirFieldName = _("Mount Point:");
		tConfigItem.stLabel = gpoMountData->getLabel();
		tConfigItem.stSrc = gpoMountData->getSrc();
		tConfigItem.stCommand = getFilterName();
		
		VolumeDialog oDlg(tConfigItem);
		
		oDlg.setMountData(gpoMountData);
		
		if (oDlg.run() != Gtk::RESPONSE_OK)
		{
			oDlg.hide();
		
			while (Gtk::Main::events_pending()) // force redraw
				Gtk::Main::iteration();
			
			cancelDataReq();
			return;
		}

		gpoMountData->setDir(oDlg.getSelectDir());
					
		oDlg.hide();
		
		// save default destination
		QConfig::getInstance().setDfDir(
			Glib::path_get_dirname(oDlg.getSelectDir()));

		while (Gtk::Main::events_pending()) // force redraw
			Gtk::Main::iteration();
	}
	
	Filter::askData();
}

//==============================================================================

bool MountFilter::attach()
{	
	assert(gpoMountData);
	
	int ci = 0;
	Glib::ustring stFSys = gpoMountData->getFSys();
	Glib::ustring stFSysDetect;
	unsigned int uiFlags = gpoMountData->getMountFlags();
	
	// reset error
	gstLastError.clear();
	
	if (isAttached())
		return false;

	// ask destination
	while (true)
	{
		if (!isValidMountPoint())
		{
			requestData();	

			if (!waitData())
				return false;
		}
		else
			break;
	}
	
	// get filesystem	
	if (!MountUtil::getFSys(gpoMountData->getSrc(), &stFSysDetect, &gstLastError))
	{
		if (stFSys.empty())
			return false;
	}
	
	if (stFSys == "fat16" || stFSys == "fat32")
		stFSys = "vfat";
	
	// flags
	if (gpoMountData->getReadOnly())
		uiFlags |= MS_RDONLY;
		
	// mount
	for (ci=0; ci<MOUNTFILTERBASE_MAXTRY; ++ci)
	{	
		if (MountUtil::mount(   gpoMountData->getSrc(),
								gpoMountData->getDir(),
								stFSys,
								uiFlags,
								&gstLastError))
		{
			gpoMountData->setFSys(stFSys);
					
			return true;
		}
		else if (stFSysDetect != stFSys && !stFSysDetect.empty())
			stFSys = stFSysDetect;
		
		Glib::usleep(MOUNTFILTERBASE_SLEEP_TRY);
	}
				
	return false;
}

//==============================================================================

bool MountFilter::detach()
{
	assert(gpoMountData);

	int ci = 0;
	
	// reset error
	gstLastError.clear();
	
	if (!isAttached())
		return false;
	
	// device is not properly mounted
	if (!MountUtil::isMounted(	gpoMountData->getSrc(),
								gpoMountData->getDir()))
	{
		Glib::ustring stDir;
		
		if (MountUtil::getMountPoint(	gpoMountData->getSrc(),
										&stDir))
			gpoMountData->setDir(stDir);
		
		else
		{
			gstLastError = _("Mount point not found.");
			return false;
		}
	}
	
	for (ci=0; ci<MOUNTFILTERBASE_MAXTRY; ++ci)
	{	
		if (MountUtil::umount(gpoMountData->getDir(), &gstLastError))
			return true;
				
		Glib::usleep(MOUNTFILTERBASE_SLEEP_TRY);
	}
				
	return false;
}

//==============================================================================

bool MountFilter::isAttached()
{
	assert(gpoMountData);
	
	if (MountUtil::isMountedSrc(gpoMountData->getSrc())
		|| MountUtil::isMountedDir(gpoMountData->getSrc()))
	{
		return true;
	}
	
	return false;
}

//==============================================================================

bool MountFilter::isValidMountPoint()
{
	assert(gpoMountData);
	assert(gpoSrcPathData);
	
	if (gpoMountData->getDir().empty())
		return false;
	
	// if src contains the mount point the source would not be available for probing
	if (gpoMountData->getDir().find(gpoSrcPathData->getSrc()) != std::string::npos)
		return false;
	
	return true;
}

//==============================================================================

bool MountFilter::probe()
{
	assert(gpoMountData);
	
	Glib::ustring stFSys = gpoMountData->getFSys();
	
	if (!isAttached())
		return false;

	if (stFSys.empty())
	{
		if (!MountUtil::getFSys(gpoMountData->getSrc(), &stFSys, &gstLastError))
			return false;
			
		gpoMountData->setFSys(stFSys);
	}
			
	return true;
}

//==============================================================================

void MountFilter::setBasePathSrc(Data *poData)
{
	gpoSrcPathData = poData;
}

//==============================================================================

void MountFilter::setSrcData(MountData *poData)
{
	gpoMountData = poData;

	Filter::setSrcData(poData);
}

//==============================================================================

} // Pline
