/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MountUtil.h"
#include "QDebug.h"
#include "../Util.h"

#include <glibmm/i18n.h>
#include <glibmm/fileutils.h>
#include <sys/mount.h>
#include <sys/param.h>
#include <errno.h>
#include <sys/utsname.h>

namespace Pline
{

#define FSYSMOD_PATH_FMT		"/lib/modules/%s/kernel/fs"
#define PROCMOUNTS_PATH			"/proc/mounts"
#define DF_PATH					"/bin/df"
#define VOLID_PATH				"/sbin/vol_id"
#define BLKID_PATH				"/sbin/blkid"
#define PROCFILESYSTEM_PATH		"/proc/filesystems"

//==============================================================================

MountUtil::MountUtil()
{
}

//==============================================================================

MountUtil::~MountUtil()
{
}

//==============================================================================

bool MountUtil::getAvailFSysMod(std::vector<Glib::ustring> *pvecMod)
{
	Glib::ustring stFSysModPath = getFSysModPath();
	
	if (stFSysModPath.empty())
		return false;
		
	Glib::Dir dir(stFSysModPath);
	
	std::vector<Glib::ustring> vecEntry(dir.begin(), dir.end());
	
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi<vecEntry.size();
			++xi)
	{
		// build full path
		Glib::ustring stMod = Glib::build_filename(stFSysModPath, vecEntry[xi]);
		
		if (stMod.find("nls") != std::string::npos)
			continue;
			
		if (stMod.size()>3 && stMod.substr(stMod.size() - 3) == ".ko")
			pvecMod->push_back(stMod);
		else if (isValidDir(stMod)) // find in first sub-dir
		{
			Glib::Dir subDir(stMod);
			std::vector<Glib::ustring> vecSubEntry(subDir.begin(), subDir.end());
			
			for (   std::vector<Glib::ustring>::size_type xiSub = 0;
					xiSub<vecSubEntry.size();
					++xiSub)
			
			{
				// buil full path
				Glib::ustring stModSub = Glib::build_filename(stMod, vecSubEntry[xiSub]);
				
				if (stModSub.substr(stModSub.size() - 3) == ".ko")
					pvecMod->push_back(stModSub);
			}
		}
	}
 
	return (pvecMod->size() > 0) ? true : false;
}

//==============================================================================

bool MountUtil::getDiskUsage(   const Glib::ustring& stDir,
								Glib::ustring *pstDiskSize,
								Glib::ustring *pstSpaceUsed,
								Glib::ustring *pstSpaceAvailable,
								int *piDiskUsagePct,
								Glib::ustring *pstErrMsg)
{
	Glib::ustring stDfCmd = DF_PATH;
	
	Glib::ustring stCmd;
	Glib::ustring stOutBuf;
	Glib::ustring stOutBuf2;
	
	size_t tPos;
	
	Glib::ustring stSize;
	Glib::ustring stUsed;
	Glib::ustring stAvail;
	Glib::ustring stUsePct;
	bool bSkipSpace = false;
	bool bSkipInfo = false;
	int ciInfo = 0;
	
	if (!isValidDir(stDir, pstErrMsg))
		return false;
	
	// init params-in
	pstDiskSize->clear();
	pstSpaceUsed->clear();
	pstSpaceAvailable->clear();
	*piDiskUsagePct = 0;
	
	if (!Util::isFileExist(stDfCmd, pstErrMsg))
		return false;
		
	stCmd = Util::compose( "%s -h \"%s\"",
							stDfCmd.c_str(),
							stDir.c_str());
	
	if (!Util::exec(stCmd, &stOutBuf))
		return false;
		
	stOutBuf2 = stOutBuf;
		
	if (((tPos = stOutBuf2.find('\n')) != std::string::npos))
	{
		stOutBuf2 = stOutBuf2.substr(tPos+1);
			
		for (unsigned int xui=0; xui<stOutBuf2.size(); xui++)
		{
			if (bSkipSpace)
			{
				if (stOutBuf2[xui] != ' ')
				{
					bSkipSpace = false;
					bSkipInfo = false;
					ciInfo++;
				}
			}
				
			if (!bSkipSpace && !bSkipInfo)
			{
				if (xui != 0)
				{
					if (ciInfo == 1)
					{
						stSize = stOutBuf2.substr((size_t)xui);
						bSkipInfo = true;
					}
					else if (ciInfo == 2)
					{
						stUsed = stOutBuf2.substr((size_t)xui);
						bSkipInfo = true;
					}
					else if (ciInfo == 3)
					{
						stAvail = stOutBuf2.substr((size_t)xui);
						bSkipInfo = true;
					}
					else if (ciInfo == 4)
					{
						stUsePct = stOutBuf2.substr((size_t)xui);
						bSkipInfo = true;
					}
				}
			}
			
			if (stOutBuf2[xui] == ' ')
				bSkipSpace = true;
		}
			
		*pstDiskSize = stSize.substr(0, stSize.find(' '));
		*pstSpaceUsed = stUsed.substr(0, stUsed.find(' '));
		*pstSpaceAvailable = stAvail.substr(0, stAvail.find(' '));
		*piDiskUsagePct = atoi(stUsePct.substr(0, stUsePct.find(' ')-1).c_str());

		return true;
	}
		
	return false;
}
								
//==============================================================================

bool MountUtil::getFSys(const Glib::ustring& stSrc, 
						Glib::ustring *pstFSys,
						Glib::ustring *pstErrMsg)
{
	Glib::ustring stVolIdCmd = VOLID_PATH;
	Glib::ustring stBlkIdCmd = BLKID_PATH;
	
	Glib::ustring stCmd;
	Glib::ustring stOutBuf;
	
	Glib::ustring stRe;

	pstFSys->clear();

	if (!isValidSrc(stSrc, pstErrMsg))
		return false;
	
	if (Util::isFileExist(stVolIdCmd, pstErrMsg))
	{
		stCmd = Util::compose( "%s %s 2>&1", 
								stVolIdCmd.c_str(),
								stSrc.c_str());
		
		stRe = "(ID_FS_TYPE)=([^\n]*)";
	}
	else if (Util::isFileExist(stBlkIdCmd, pstErrMsg))
	{
		stCmd = Util::compose( "%s %s 2>&1", 
								stBlkIdCmd.c_str(),
								stSrc.c_str());
		
		stRe = "(TYPE)=\"([^\"]*)\"";
	}
	else
	{
		if (pstErrMsg)
			*pstErrMsg = _("\"vol_id\" not found. \"blkid\" not found.");
			
		return false;
	}

	if (!Util::exec(stCmd, &stOutBuf))
	{
		if (pstErrMsg)
			*pstErrMsg = _("Filesystem unknown.");
			
		return false;
	}
		
	Glib::RefPtr<Glib::Regex> re = 
			Glib::Regex::create(stRe);
	
	if (re->match(stOutBuf))
	{
		std::vector<Glib::ustring> vec = re->split(stOutBuf);
			
		for (   std::vector<Glib::ustring>::size_type xi = 0;
				xi<vec.size();
				++xi)
		{
			Glib::ustring stToken = vec[xi].lowercase();
			
			if ((stToken == "type"  || stToken == "id_fs_type")
				&& xi < vec.size()-1)
			{
				*pstFSys = vec[xi+1];
				return true;
			}
		}	
	}
	
	if (pstErrMsg)
		*pstErrMsg = _("Filesystem unknown.");
	
	return false;
}

//==============================================================================

bool MountUtil::getFSysList(std::vector<Glib::ustring> *pvecFSys)
{
	std::vector<Glib::ustring> vecFSys;
	Glib::ustring stOutBuf;
	Glib::ustring stRe = "(.*?)[\\t ]+(.+?)\\R";
	unsigned int xui;

	if (!Util::file2string(PROCFILESYSTEM_PATH, &stOutBuf))
		return false;
	
	Glib::RefPtr<Glib::Regex> re = Glib::Regex::create(stRe);
	
	if (re->match(stOutBuf))
	{
		std::vector<Glib::ustring> vec = re->split(stOutBuf);
			
		int ciEmpty = 0;
		for (   std::vector<Glib::ustring>::size_type xi = 0;
				xi<vec.size();
				++xi)
		{
			Glib::ustring stToken = vec[xi];
			
			if (stToken == "nodev")
			{
				ciEmpty = 0;
				continue;
			}
				
			if (ciEmpty == 2)
			{
				if (!stToken.empty())
					vecFSys.push_back(stToken);
					
				ciEmpty = 0;
			}
			else if (vec[xi].empty())
				ciEmpty++;
		}	
	}
	
	sort(vecFSys.begin(), vecFSys.end(), Util::cmpst);

	for (xui=0; xui<vecFSys.size(); xui++)
		pvecFSys->push_back(vecFSys[xui]);

	return false;
}

//==============================================================================

const Glib::ustring MountUtil::getFSysModPath()
{
	struct utsname tBuf;
	Glib::ustring stSysRel;
	
	if (uname(&tBuf) == 0)
	{
		stSysRel = tBuf.release;
	
		return Util::compose(FSYSMOD_PATH_FMT, stSysRel.c_str());
	}
	
	return "";
}

//==============================================================================

bool MountUtil::getMountPoint(	const Glib::ustring& stSrc,
								Glib::ustring *pstDir)
{
	Glib::ustring stUnEscapedDir;
	Glib::ustring stRe;
	Glib::ustring stMounts;
			
	if (stSrc.empty())
		return false;
		
	if (!Util::file2string(PROCMOUNTS_PATH, &stMounts))
		return false;
	
	if (stMounts.empty())
		return false;

	stRe = Util::compose("(%s) (.*?) ", stSrc.c_str());
	Glib::RefPtr<Glib::Regex> re = 	
		Glib::Regex::create(stRe);
	
	if (re->match(stMounts))
	{
		std::vector<Glib::ustring> vec = re->split(stMounts);
		
		for (   std::vector<Glib::ustring>::size_type xi = 0;
				xi<vec.size();
				++xi)
		{
			if (vec[xi] == stSrc && (xi+1) < vec.size())
			{
				*pstDir = vec[xi+1];
				return true;
			}
		}
	}
	
	return false;
}

//==============================================================================

bool MountUtil::isMounted(  const Glib::ustring& stSrc, 
							const Glib::ustring& stDir)
{
	Glib::ustring stEscapedDir = stDir;
	Glib::ustring stMatch;
	Glib::ustring stMounted;
	size_t iPos;
			
	if (stDir.empty())
		return false;
		
	if (!Util::file2string(PROCMOUNTS_PATH, &stMounted))
		return false;
	
	if (stMounted.empty())
		return false;

	while ((iPos = stEscapedDir.find(" ")) != std::string::npos)
		stEscapedDir.replace(iPos, 1, Util::compose("\\0%o", ' '));
	
	stMatch = Util::compose("%s %s ", stSrc.c_str(), stEscapedDir.c_str());

	return (stMounted.find(stMatch) != std::string::npos) ? true : false;
}

//==============================================================================

bool MountUtil::isMountedDir(const Glib::ustring& stDir)
{
	Glib::ustring stEscapedDir = stDir;
	Glib::ustring stMatch;
	Glib::ustring stMounts;
	size_t iPos;
			
	if (stDir.empty())
		return false;
		
	if (!Util::file2string(PROCMOUNTS_PATH, &stMounts))
		return false;
	
	if (stMounts.empty())
		return false;

	while ((iPos = stEscapedDir.find(" ")) != std::string::npos)
		stEscapedDir.replace(iPos, 1, Util::compose("\\0%o", ' '));
	
	stMatch = Util::compose("%s ", stEscapedDir.c_str());

	return (stMounts.find(stMatch) != std::string::npos) ? true : false;
}

//==============================================================================

bool MountUtil::isMountedSrc(const Glib::ustring& stSrc)
{
	Glib::ustring stMatch;
	Glib::ustring stMounts;
			
	if (stSrc.empty())
		return false;
		
	if (!Util::file2string(PROCMOUNTS_PATH, &stMounts))
		return false;
	
	if (stMounts.empty())
		return false;
	
	stMatch = Util::compose("%s ", stSrc.c_str());

	return (stMounts.find(stMatch) != std::string::npos) ? true : false;
}

//==============================================================================

bool MountUtil::isValidLabel(   const Glib::ustring& stLabel, 
								Glib::ustring *pstErrMsg)
{
	const char achOK[] = {' ', '-', '.', '@', '_', '\0'};
	unsigned int xui, cui;
	char chOK, chDir;
	bool bOK = true;
	bool bChOK = false;

	if (stLabel.empty())
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid Volume Label");
			
		return false;
	}

	for (xui=0; xui<stLabel.size(); ++xui)
	{
		chDir = stLabel[xui];

		if ((chDir >= 'a' && chDir <= 'z')
			|| (chDir >= 'A' && chDir <= 'Z')
			|| (chDir >= '0' && chDir <= '9'))
		{
			bChOK = true;
		}
		else
		{
			cui = 0;	
			while (true)
			{
				if ((chOK = achOK[cui]) == '\0')
					break;
		
				if (chDir == chOK)
				{
					bChOK = true;
					break;
				}

				cui++;
			}
		}
	
		if (!bChOK)
		{
			bOK = false;
			break;
		}
		else
			bChOK = false;
	}

	if (!bOK)
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid characters found in Volume Label. Allowed characters: -, ., @, _");
	}
	
	return bOK;
}

//==============================================================================

bool MountUtil::isValidDir( const Glib::ustring& stDir, 
							Glib::ustring *pstErrMsg)
{
	if (!Util::isFileExist(stDir, pstErrMsg))			
		return false;
	
	if (!Util::isDirectory(stDir, pstErrMsg))
		return false;
	
	return true;
}

//==============================================================================

bool MountUtil::isValidFSys(const Glib::ustring& stFSys, 
							Glib::ustring *pstErrMsg)
{
	if (stFSys.empty())
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid filesystem.");
			
		return false;
	}

	return true;
}

//==============================================================================

bool MountUtil::isValidSrc( const Glib::ustring& stSrc, 
							Glib::ustring *pstErrMsg)
{
	if (!Util::isFileExist(stSrc, pstErrMsg))
		return false;
	
	if (!Util::isDevice(stSrc, pstErrMsg))
		return false;
	
	return true;
}
	
//==============================================================================

bool MountUtil::mount(  const Glib::ustring& stSrc, 
						const Glib::ustring& stDir,
						const Glib::ustring& stFSys, 
						unsigned long ulMountFlags,
						Glib::ustring* pstErrMsg)
{
	Glib::ustring stErrMsg;
	int iMount = 0;
	
	if (pstErrMsg)
		pstErrMsg->clear();

	// check input params
	if (!isValidSrc(stSrc, pstErrMsg))
		return false;
	
	if (!isValidDir(stDir, pstErrMsg))
		return false;
	
	if (!isValidFSys(stFSys, pstErrMsg))
		return false;
		
	// mount	
	iMount = ::mount(   stSrc.c_str(), 
						stDir.c_str(),
						stFSys.c_str(),
						ulMountFlags,
						(const void*)NULL);
	
	if (iMount == 0)
	{
		debug_message(DEBUG_PLINE, "src [%s], dir [%s]", stSrc.c_str(), stDir.c_str());
		
		return true;
	}
	else
	{
		switch (errno)
		{
			case EACCES:
				stErrMsg = _("A component of a path was not searchable.");
				break;
			case EAGAIN:
				stErrMsg = _("A call to umount2 specifying MNT_EXPIRE successfully marked an unbusy file system as expired.");
				break;
			case EBUSY:
				stErrMsg = Util::compose(_("%s already mounted."), stSrc.c_str());
				break;
			case EFAULT:
				stErrMsg = _("One of the pointer arguments points outside the user address space.");
				break;
			case EINVAL:
				stErrMsg = Util::compose(_("%s not recognized for the specified filesystem."), stSrc.c_str());
				break;
			case ELOOP:
				stErrMsg = _("Too many link encountered during pathname resolution.");
				break;
			case EMFILE:
				stErrMsg = _("Table of dummy devices is full.");
				break;
			case ENAMETOOLONG:
				stErrMsg = Util::compose(_("Source or destination pathname can't be longer than %d characters."), MAXPATHLEN);
				break;
			case ENODEV:
				stErrMsg = Util::compose(_("Filesystem type %s not configured in the kernel."), stFSys.c_str());
				break;
			case ENOENT:
				stErrMsg = _("Source or destination might not exist.");
				break;
			case ENOMEM:
				stErrMsg = _("The kernel could not allocate a free page to copy filenames or data into.");
				break;
			case ENOTBLK:
				stErrMsg = Util::compose(_("%s is not a block device (and a device was required)."), stSrc.c_str());
				break;
			case ENOTDIR:
				stErrMsg = Util::compose(_("%s not a directory."), stDir.c_str());
				break;
			case ENXIO:
				stErrMsg = _("The major number of the block device source is out of range.");
				break;
			case EPERM:
				stErrMsg = _("The caller does not have the required privileges.");
				break;
			default:
				stErrMsg = "";
				break;
		}
		
		if (pstErrMsg)
			*pstErrMsg = stErrMsg;
			
		return false;
	}
}

//==============================================================================

bool MountUtil::umount(const Glib::ustring& stDir, Glib::ustring* pstErrMsg)
{
	Glib::ustring stErrMsg;
	int iUmount = 0;
	
	if (pstErrMsg)
		pstErrMsg->clear();

	// check input params
	if (!isValidDir(stDir, pstErrMsg))
		return false;
		
	iUmount = ::umount(stDir.c_str());
	
	if (iUmount == 0)
	{
		debug_message(DEBUG_PLINE, "dir [%s]", stDir.c_str());
		
		return true;
	}
	else
	{
		switch (errno)
		{
			case EAGAIN:
				stErrMsg = _("A call to umount specifying MNT_EXPIRE successfully marked an unbusy file system as expired.");
				break;
			case EBUSY:
				stErrMsg = Util::compose(_("%s is busy."), stDir.c_str());
				break;
			case EFAULT:
				stErrMsg = _("Destination points outside the user address space.");
				break;
			case EINVAL:
				stErrMsg = Util::compose(_("%s is not a mount point."), stDir.c_str());
				break;
			case ENAMETOOLONG:
				stErrMsg = Util::compose(_("Source or destination pathname can't be longer than %d characters."), MAXPATHLEN);
				break;
			case ENOENT:
				stErrMsg = _("Source or destination might not exist.");
				break;
			case ENOMEM:
				stErrMsg = _("The kernel could not allocate a free page to copy filenames or data into.");
				break;
			case EPERM:
				stErrMsg = _("The caller does not have the required privileges.");
				break;
			default:
				stErrMsg = "";
				break;
		}
		
		if (pstErrMsg)
			*pstErrMsg = stErrMsg;
		
		return false;
	}
}

//==============================================================================

} // Pline	
