/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "PermFilter.h"
#include "../QDebug.h"
#include "../Util.h"
#include "MountData.h"

#include <glibmm/i18n.h>
#include <assert.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

namespace Pline
{

//==============================================================================

PermFilter::PermFilter()
: Filter()
{
	gpoMountData = 0;
}

//==============================================================================

PermFilter::~PermFilter()
{
}

//==============================================================================

bool PermFilter::run()
{
	assert(gpoMountData);
	
	// reset error
	gstLastError.clear();

	if (!gpoMountData->getDirSetPerm())
		return true;
		
	// data
	Glib::ustring stDir = gpoMountData->getDir();
	Glib::ustring stOwner = gpoMountData->getDirOwner();
	Glib::ustring stGroup = gpoMountData->getDirGroup();
	mode_t tPerm = (mode_t)gpoMountData->getDirPerm();
	
	// check data
	if (!Util::isDirectory(stDir, &gstLastError))
		return false;
	
	if (stOwner.empty())
	{
		gstLastError = _("Invalid owner.");
		
		return false;
	}
	
	if (stGroup.empty())
	{
		gstLastError = _("Invalid group.");
		
		return false;
	}
	
	// find group id, user id
	std::vector<Glib::ustring> vecGroupName;
	std::vector<int> vecGroupID;
	std::vector<Glib::ustring> vecUserName;
	std::vector<int> vecUserID;
	
	bool bGroupIDFound = false;
	bool bUserIDFound = false;
	
	gid_t tGroupID;
	uid_t tUserID;
	
	if (!Util::getGroups(	&vecGroupName,
							&vecGroupID,
							&gstLastError))
	{
		return false;
	}
	
	if (!Util::getUsers(	&vecUserName,
							&vecUserID,
							&gstLastError))
	{
		return false;
	}
	
	for (	std::vector<Glib::ustring>::size_type xi = 0;
			xi < vecGroupName.size();
			++xi)
	{
		if (vecGroupName[xi] == stGroup)
		{
			tGroupID = (gid_t)vecGroupID[xi];
			
			bGroupIDFound = true;
			break;
		}
	}
	
	if (!bGroupIDFound)
	{
		gstLastError = Util::compose(	_("Group %s not found."), 
										stGroup.c_str());
		return false;
	}
	
	for (	std::vector<Glib::ustring>::size_type xi = 0;
			xi < vecUserName.size();
			++xi)
	{
		if (vecUserName[xi] == stOwner)
		{
			tUserID = (uid_t)vecUserID[xi];
			
			bUserIDFound = true;
			break;
		}
	}
	
	if (!bUserIDFound)
	{
		gstLastError = Util::compose(	_("User %s not found."), 
										stOwner.c_str());
		return false;
	}
	
	// apply changes
	int iErr = 0;
	bool bOwnerSet = false;
	bool bPermSet = false;
	Glib::ustring stChOwnError;
	Glib::ustring stChModError;
	
	// apply chown
	if ((iErr = chown(	stDir.c_str(), 
						tUserID,
						tGroupID)) == 0)
	{
		bOwnerSet = true;
	}
	else
	{
		switch (errno)
		{
			case EACCES:
				stChOwnError = _("Search permission is denied on a component of the path prefix.");
				break;
			case EFAULT:
				stChOwnError = _("Path points outside your accessible address space.");
				break;
			case ELOOP:
				stChOwnError = _("Too many symbolic links were encountered in resolving path.");
				break;
			case ENAMETOOLONG:
				stChOwnError = _("Path is too long.");
				break;
			case ENOENT:
				stChOwnError = _("File does not exist.");
				break;
			case ENOMEM:
				stChOwnError = _("Insufficient kernel memory was available.");
				break;	
			case ENOTDIR:
				stChOwnError = _("A component of the path prefix is not a directory.");
				break;
			case EPERM:
				stChOwnError = _("The calling process did not have the required permissions to change owner and/or group.");
				break;
			case EROFS:
				stChOwnError = _("Can't change ownership to a read-only file system.");
				break;
			default:
				stChOwnError = _("Unknown error.");
				break;
		}
	}

	// apply permissions
	struct stat tBuf;
	mode_t tPermOri = tPerm;
	
	if (stat(stDir.c_str(), &tBuf) != 0)
	{
		gstLastError = _("Failed reading permissions.");
		return false;
	}
	
	if (tPermOri & S_IRUSR)
		tPerm |= S_IXUSR;
	if (tPermOri & S_IWUSR)
		tPerm |= S_IRUSR|S_IXUSR;
	
	if (tPermOri & S_IRGRP)
		tPerm |= S_IXGRP;
	if (tPermOri & S_IWGRP)
		tPerm |= S_IRGRP|S_IXGRP;

	if (tPermOri & S_IROTH)
		tPerm |= S_IXOTH;
	if (tPermOri & S_IWOTH)
		tPerm |= S_IROTH|S_IXOTH;
	
	if ((tPerm|S_IFDIR) == tBuf.st_mode
	    || chmod(stDir.c_str(), tPerm) == 0)
	{
		bPermSet = true;
	}
	else
	{
		switch(errno)
		{
			case EACCES:
				stChModError = _("Search permission is denied on a component of the path prefix.");
				break;
			case EFAULT:
				stChModError = _("Path points outside your accessible address space.");
				break;
			case EIO:
				stChModError = _("An I/O error occurred.");
				break;
			case ELOOP:
				stChModError = _("Too many symbolic links were encountered in resolving path.");
				break;
			case ENAMETOOLONG:
				stChModError = _("Path is too long.");
				break;
			case ENOENT:
				stChModError = _("The file does not exist.");
				break;
			case ENOMEM:
				stChModError = _("Insufficient kernel memory was available.");
				break;
			case ENOTDIR:
				stChModError = _("A component of the path prefix is not a directory.");
				break;
			case EPERM:
				stChModError = _("The effective UID does not match the owner of the file, and the process is not privileged.");
				break;
			case EROFS:
				stChModError = _("Can't change permissions to a read-only file system.");
				break;
			default:
				stChModError = _("Unknown error.");
				break;
		}
	}
	
	// Set Error
	if (bOwnerSet && bPermSet)
		return true;
	else if (!bOwnerSet && bPermSet)
		gstLastError = _("Error setting ownership: ") + stChOwnError;
	else if (bOwnerSet && !bPermSet)
		gstLastError = _("Error changing permissions: ") + stChModError;
	else if (!bOwnerSet && !bPermSet)
		gstLastError = Util::compose(	_("Ownership: %s\nPermissions: %s"), 
										stChOwnError.c_str(), 
										stChModError.c_str());
	
	return false;
}

//==============================================================================

void PermFilter::setSrcData(MountData *poData)
{
	gpoMountData = poData;
	
	Filter::setSrcData(poData);
}

//==============================================================================

} // Pline
