/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "eMountSh.h"
#include "QDebug.h"
#include <QConfig.h>
#include <config.h>
#include "eMountSys.h"
#include "Pline/MountPline.h"
#include "VolumeConfig.h"

#include <iostream>
#include <glibmm/i18n.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>

using std::cout;
using std::cerr;

eMountSh::eMountSh()
{
	gpoVolumeConfig = 0;
}

//==============================================================================

eMountSh::~eMountSh()
{
	if (gpoVolumeConfig)
		delete gpoVolumeConfig;
}

//==============================================================================

eMountSh& eMountSh::getInstance()
{
	static eMountSh oEmountSh;
	
	return oEmountSh;
}

//==============================================================================

bool eMountSh::eject()
{
	if (!gpoVolumeConfig)
		return false;
	
	// create pline
	Pline::MountPline oMount(Pline::kNoSignalDataMode, gpoVolumeConfig);
	oMount.setRunMode(Pline::MountPline::kRunEject);
	
	// check pline status
	if (!oMount.isInUse())
	{
		cerr << _("Volume not mounted.") << std::endl;
		
		return false;
	}
	else
	{
		if (oMount.run())
		{
			cout << _("Volume ejected.") << std::endl;
			return true;
		}
		else
		{
			cerr	<< _("Volume could not be ejected (")
					<< oMount.getLastError().stErrMsg
					<< _(")")
					<< std::endl;
					
			return false;
		}
	}
}

//==============================================================================

bool eMountSh::findVolumeConfig(Glib::ustring stVolumeLabel)
{
	VolumeConfig oVolumeConfig;
	
	// delete existing volumeconfig
	if (gpoVolumeConfig)
	{
		delete gpoVolumeConfig;
		gpoVolumeConfig = 0;
	}

	// get volumeconfig
	if (!QConfig::getInstance().findVolume(stVolumeLabel, oVolumeConfig))
	{
		cerr << _("Volume not found.") << std::endl;
		
		return false;
	}
	
	// create new volumeconfig copy
	gpoVolumeConfig = new VolumeConfig;
	*gpoVolumeConfig = oVolumeConfig;
	
	return true;
}

//==============================================================================

bool eMountSh::mount()
{
	bool bMounted = false;
	
	if (!gpoVolumeConfig)
		return false;
	
	// create pline
	Pline::MountPline oMount(Pline::kNoSignalDataMode, gpoVolumeConfig);
	oMount.setRunMode(Pline::MountPline::kRunMount);
		
	// check pline status
	if (oMount.isInUse(&bMounted) && bMounted)
	{
		cerr << _("Volume already mounted.") << std::endl;
		
		return false;
	}
	else
	{
		if (oMount.run())
		{				
			cout << _("Volume mounted.") << std::endl;
			return true;
		}
		else
		{
			cerr	<< _("Volume could not be mounted (") 
					<< oMount.getLastError().stErrMsg
					<< _(")")
					<< std::endl;
					
			return false;
		}
	}
}

//==============================================================================

bool eMountSh::ping()
{
	bool bMounted = false;
	
	if (!gpoVolumeConfig)
		return false;
	
	// create pline
	Pline::MountPline oMount(Pline::kNoSignalDataMode, gpoVolumeConfig);

	return (oMount.isInUse(&bMounted) && bMounted) ? true : false;
}
	
//==============================================================================

int eMountSh::run(  const OptionGroup::eMode eMode,
					Glib::ustring& stVolumeLabel)
{
	int iRet;

	switch(eMode)
	{
		case OptionGroup::kModeVersion:
			iRet = showVersion() ? 0 : 1;
			break;
		case OptionGroup::kModeEject:
			iRet = (findVolumeConfig(stVolumeLabel) && eject()) ? 0 : 1;
			break;
		case OptionGroup::kModeList:
			iRet = showList() ? 0 : 1;
			break;
		case OptionGroup::kModeMount:
			iRet = (findVolumeConfig(stVolumeLabel) && mount()) ? 0 : 1;
			break;
		case OptionGroup::kModePing:
			iRet = (findVolumeConfig(stVolumeLabel) && ping()) ? 0 : 1;
			break;
		case OptionGroup::kModeStatus:
			iRet = (findVolumeConfig(stVolumeLabel) && showStatus()) ? 0 : 1;
			break;	
		default:
			iRet = 1;
			break;
	}
		
	return iRet;
}

//==============================================================================

bool eMountSh::showList()
{
	std::list<VolumeConfig*> lstVolume = QConfig::getInstance().getVolumes();

	lstVolume.sort(VolumeConfig::cmpByPos);
		
	for (   std::list<VolumeConfig*>::iterator ite = lstVolume.begin();
			ite != lstVolume.end();
			++ite)
	{
		cout << ((VolumeConfig*)*ite)->getData().getLabel() << std::endl;
	}
		
	return false;
}

//==============================================================================

bool eMountSh::showStatus()
{
	if (!gpoVolumeConfig)
		return false;
			
	// create pline
	Pline::MountPline oMount(Pline::kNoSignalDataMode, gpoVolumeConfig);
	oMount.setRunMode(Pline::MountPline::kRunDumpStatus);
	
	// check pline status
	if (!oMount.isInUse())
	{
		cerr << _("Volume not mounted.") << std::endl;
		
		return false;
	}
	else
	{
		bool bRun = oMount.run();
		
		std::vector<Pline::MountPline::tDumpStatus> vecDumpStatus =
			oMount.getDumpStatus();
		
		Glib::ustring stCipher = oMount.getCipher();
		
		for (   std::vector<Pline::MountPline::tDumpStatus>::size_type xi = 0;
				xi < vecDumpStatus.size();
				++xi)
		{
			if (vecDumpStatus[xi].stFilterName == "Crypt"
				&& !stCipher.empty())
			{
				cout	<< vecDumpStatus[xi].stFilterName
						<< " "
						<< vecDumpStatus[xi].stSrcType
						<< " "
						<< vecDumpStatus[xi].stSrc
						<< ". "
						<< stCipher
						<< std::endl;
			}
			else
			{
				cout	<< vecDumpStatus[xi].stFilterName
						<< " "
						<< vecDumpStatus[xi].stSrcType
						<< " "
						<< vecDumpStatus[xi].stSrc
						<< std::endl;
			}
		}
		
		return bRun;
	}
}

//==============================================================================

bool eMountSh::showVersion()
{
	std::cout << VERSION << std::endl;
	
	return true;
}

//==============================================================================
